@@ -1,6 +1,7 @@
 #[cfg(with_options)]
 use crate::fs::OpenOptions;
 use crate::fs::{Metadata, Permissions};
+use cap_primitives::{ambient_authority, AmbientAuthority};
 #[cfg(target_os = "wasi")]
 use std::path::Path;
 use std::{
@@ -35,13 +36,13 @@ pub struct File {
 impl File {
     /// Constructs a new instance of `Self` from the given [`std::fs::File`].
     ///
-    /// # Safety
+    /// # Ambient Authority
     ///
     /// [`std::fs::File`] is not sandboxed and may access any path that the
     /// host process has access to.
     #[inline]
-    pub unsafe fn from_std(std: fs::File) -> Self {
-        Self::from_cap_std(crate::fs::File::from_std(std))
+    pub fn from_std(std: fs::File, ambient_authority: AmbientAuthority) -> Self {
+        Self::from_cap_std(crate::fs::File::from_std(std, ambient_authority))
     }
 
     /// Constructs a new instance of `Self` from the given `cap_std::fs::File`.
@@ -121,7 +122,7 @@ impl File {
 impl FromRawFd for File {
     #[inline]
     unsafe fn from_raw_fd(fd: RawFd) -> Self {
-        Self::from_std(fs::File::from_raw_fd(fd))
+        Self::from_std(fs::File::from_raw_fd(fd), ambient_authority())
     }
 }
 
@@ -129,7 +130,7 @@ impl FromRawFd for File {
 impl FromRawHandle for File {
     #[inline]
     unsafe fn from_raw_handle(handle: RawHandle) -> Self {
-        Self::from_std(fs::File::from_raw_handle(handle))
+        Self::from_std(fs::File::from_raw_handle(handle), ambient_authority())
     }
 }
 
