@@ -1,5 +1,7 @@
 use crate::net::{Shutdown, SocketAddr};
+use cap_primitives::{ambient_authority, AmbientAuthority};
 use std::{
+    fmt,
     io::{self, IoSlice, IoSliceMut, Read, Write},
     net,
     time::Duration,
@@ -18,11 +20,11 @@ use {
 /// This corresponds to [`std::net::TcpStream`].
 ///
 /// Note that this `TcpStream` has no `connect` method. To create a `TcpStream`,
-/// you must first obtain a [`Catalog`] permitting the address, and then call
-/// [`Catalog::connect_tcp_stream`].
+/// you must first obtain a [`Pool`] permitting the address, and then call
+/// [`Pool::connect_tcp_stream`].
 ///
-/// [`Catalog`]: struct.Catalog.html
-/// [`Catalog::connect_tcp_stream`]: struct.Catalog.html#method.connect_tcp_stream
+/// [`Pool`]: struct.Pool.html
+/// [`Pool::connect_tcp_stream`]: struct.Pool.html#method.connect_tcp_stream
 pub struct TcpStream {
     std: net::TcpStream,
 }
@@ -30,15 +32,23 @@ pub struct TcpStream {
 impl TcpStream {
     /// Constructs a new instance of `Self` from the given `std::net::TcpStream`.
     ///
-    /// # Safety
+    /// # Ambient Authority
     ///
     /// `std::net::TcpStream` is not sandboxed and may access any address that the host
     /// process has access to.
     #[inline]
-    pub unsafe fn from_std(std: net::TcpStream) -> Self {
+    pub fn from_std(std: net::TcpStream, _: AmbientAuthority) -> Self {
         Self { std }
     }
 
+    /// Returns the socket address of the remote peer of this TCP connection.
+    ///
+    /// This corresponds to [`std::net::TcpStream::peer_addr`].
+    #[inline]
+    pub fn peer_addr(&self) -> io::Result<SocketAddr> {
+        self.std.peer_addr()
+    }
+
     /// Returns the local socket address of this listener.
     ///
     /// This corresponds to [`std::net::TcpStream::local_addr`].
@@ -61,7 +71,7 @@ impl TcpStream {
     #[inline]
     pub fn try_clone(&self) -> io::Result<Self> {
         let tcp_stream = self.std.try_clone()?;
-        Ok(unsafe { Self::from_std(tcp_stream) })
+        Ok(Self::from_std(tcp_stream, ambient_authority()))
     }
 
     /// Sets the read timeout to the timeout specified.
@@ -158,7 +168,7 @@ impl TcpStream {
 impl FromRawFd for TcpStream {
     #[inline]
     unsafe fn from_raw_fd(fd: RawFd) -> Self {
-        Self::from_std(net::TcpStream::from_raw_fd(fd))
+        Self::from_std(net::TcpStream::from_raw_fd(fd), ambient_authority())
     }
 }
 
@@ -166,7 +176,7 @@ impl FromRawFd for TcpStream {
 impl FromRawSocket for TcpStream {
     #[inline]
     unsafe fn from_raw_socket(socket: RawSocket) -> Self {
-        Self::from_std(net::TcpStream::from_raw_socket(socket))
+        Self::from_std(net::TcpStream::from_raw_socket(socket), ambient_authority())
     }
 }
 
@@ -355,4 +365,8 @@ impl Write for &TcpStream {
     }
 }
 
-// TODO: impl Debug for TcpStream
+impl fmt::Debug for TcpStream {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.std.fmt(f)
+    }
+}
