@@ -1,6 +1,7 @@
 use crate::{net::Shutdown, os::unix::net::SocketAddr};
+use cap_primitives::{ambient_authority, AmbientAuthority};
 use std::{
-    io,
+    fmt, io,
     os::unix::{
         self,
         io::{AsRawFd, FromRawFd, IntoRawFd, RawFd},
@@ -31,12 +32,12 @@ pub struct UnixDatagram {
 impl UnixDatagram {
     /// Constructs a new instance of `Self` from the given `std::os::unix::net::UnixDatagram`.
     ///
-    /// # Safety
+    /// # Ambient Authority
     ///
     /// `std::os::unix::net::UnixDatagram` is not sandboxed and may access any address that
     /// the host process has access to.
     #[inline]
-    pub unsafe fn from_std(std: unix::net::UnixDatagram) -> Self {
+    pub fn from_std(std: unix::net::UnixDatagram, _: AmbientAuthority) -> Self {
         Self { std }
     }
 
@@ -50,7 +51,7 @@ impl UnixDatagram {
     #[inline]
     pub fn unbound() -> io::Result<Self> {
         let unix_datagram = unix::net::UnixDatagram::unbound()?;
-        Ok(unsafe { Self::from_std(unix_datagram) })
+        Ok(Self::from_std(unix_datagram, ambient_authority()))
     }
 
     /// Creates an unnamed pair of connected sockets.
@@ -62,8 +63,12 @@ impl UnixDatagram {
     /// [`std::os::unix::net::UnixDatagram::pair`]: https://doc.rust-lang.org/std/os/unix/net/struct.UnixDatagram.html#method.pair
     #[inline]
     pub fn pair() -> io::Result<(Self, Self)> {
-        unix::net::UnixDatagram::pair()
-            .map(|(a, b)| unsafe { (Self::from_std(a), Self::from_std(b)) })
+        unix::net::UnixDatagram::pair().map(|(a, b)| {
+            (
+                Self::from_std(a, ambient_authority()),
+                Self::from_std(b, ambient_authority()),
+            )
+        })
     }
 
     /// Creates a new independently owned handle to the underlying socket.
@@ -74,7 +79,7 @@ impl UnixDatagram {
     #[inline]
     pub fn try_clone(&self) -> io::Result<Self> {
         let unix_datagram = self.std.try_clone()?;
-        Ok(unsafe { Self::from_std(unix_datagram) })
+        Ok(Self::from_std(unix_datagram, ambient_authority()))
     }
 
     /// Returns the address of this socket.
@@ -201,7 +206,10 @@ impl UnixDatagram {
 impl FromRawFd for UnixDatagram {
     #[inline]
     unsafe fn from_raw_fd(fd: RawFd) -> Self {
-        Self::from_std(unix::net::UnixDatagram::from_raw_fd(fd))
+        Self::from_std(
+            unix::net::UnixDatagram::from_raw_fd(fd),
+            ambient_authority(),
+        )
     }
 }
 
@@ -222,4 +230,8 @@ impl IntoRawFd for UnixDatagram {
 // Safety: `UnixDatagram` wraps a `net::UnixDatagram` which owns its handle.
 unsafe impl OwnsRaw for UnixDatagram {}
 
-// TODO: impl Debug for UnixDatagram
+impl fmt::Debug for UnixDatagram {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.std.fmt(f)
+    }
+}
