@@ -1,5 +1,7 @@
 use crate::{net::Shutdown, os::unix::net::SocketAddr};
+use cap_primitives::{ambient_authority, AmbientAuthority};
 use std::{
+    fmt,
     io::{self, IoSlice, IoSliceMut, Read, Write},
     os::unix::{
         self,
@@ -27,12 +29,12 @@ pub struct UnixStream {
 impl UnixStream {
     /// Constructs a new instance of `Self` from the given `std::os::unix::net::UnixStream`.
     ///
-    /// # Safety
+    /// # Ambient Authority
     ///
     /// `std::os::unix::net::UnixStream` is not sandboxed and may access any address that
     /// the host process has access to.
     #[inline]
-    pub unsafe fn from_std(std: unix::net::UnixStream) -> Self {
+    pub fn from_std(std: unix::net::UnixStream, _: AmbientAuthority) -> Self {
         Self { std }
     }
 
@@ -45,8 +47,12 @@ impl UnixStream {
     /// [`std::os::unix::net::UnixStream::pair`]: https://doc.rust-lang.org/std/os/unix/net/struct.UnixStream.html#method.pair
     #[inline]
     pub fn pair() -> io::Result<(Self, Self)> {
-        unix::net::UnixStream::pair()
-            .map(|(a, b)| unsafe { (Self::from_std(a), Self::from_std(b)) })
+        unix::net::UnixStream::pair().map(|(a, b)| {
+            (
+                Self::from_std(a, ambient_authority()),
+                Self::from_std(b, ambient_authority()),
+            )
+        })
     }
 
     /// Creates a new independently owned handle to the underlying socket.
@@ -57,7 +63,7 @@ impl UnixStream {
     #[inline]
     pub fn try_clone(&self) -> io::Result<Self> {
         let unix_stream = self.std.try_clone()?;
-        Ok(unsafe { Self::from_std(unix_stream) })
+        Ok(Self::from_std(unix_stream, ambient_authority()))
     }
 
     /// Returns the socket address of the local half of this connection.
@@ -154,7 +160,7 @@ impl UnixStream {
 impl FromRawFd for UnixStream {
     #[inline]
     unsafe fn from_raw_fd(fd: RawFd) -> Self {
-        Self::from_std(unix::net::UnixStream::from_raw_fd(fd))
+        Self::from_std(unix::net::UnixStream::from_raw_fd(fd), ambient_authority())
     }
 }
 
@@ -309,4 +315,8 @@ impl Write for &UnixStream {
     }
 }
 
-// TODO: impl Debug for UnixStream
+impl fmt::Debug for UnixStream {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.std.fmt(f)
+    }
+}
